function varargout = gin_clusters_plabels(varargin)
% 
% Display and analysis of SPM{.}
% FORMAT TabDat = gin_list_dlabels('List', SPM, VOL, Dis, Num, Title)
% Summary list of local maxima for entire volume of interest
%
% SPM    - structure containing SPM, distribution & filtering details
% VOL    - structure containing details of volume analysed
%
% (see spm_getSPM for further details of SPM & VOL structures)
%
% Dis    - Minimum distance between maxima
%          {defaults (missing or empty) to 8mm for 'List'}
% Num    - Maxiumum number of local maxima tabulated per cluster
%          {defaults (missing or empty) to 3   for 'List'}
% Title  - Title text for table {defaults on missing or empty}
%
% TabDat - Structure containing table data
%
% See spm_list.m for more details
%
% This function loads the labelised atlas, MNI VOIs & Labels, and the distances
% between VOIs and Labels.
% For each cluster (identified by its first local maximum), 
% this function search for the intersected regions with the cluster,
% and displays the table :
%		- local maximum for the cluster
%		- the intersected regions for the cluster.
%		- the voxels pourcents for the cluster belongs to 
% 		each of the regions (labels)
% To Compute Labels & Distances we use the 'gin_det_plabels'
% routine. 
% ______________________________________________________________________
%
% gin_clusters_plabels			B Landeau 27/12/01
% _______________________________________________________________________
% 


%-Parse arguments, default to 'list' if varargin{1} not an Action string
%-----------------------------------------------------------------------
if nargin==0, error('Insufficient arguments'), end
if ~ischar(varargin{1})
	warning('Direct usage Grandfathered: Use an action string!')
	gin_clusters_plabels('list',varargin{:});
	return
end



%=======================================================================
%switch lower(varargin{1}), case 'list'                            %-List
%=======================================================================
% FORMAT TabDat = gin_clusters_plabels('list',SPM,VOL,Dis,Num,Title,hReg)


%-Tolerance for p-value underflow, when computing equivalent Z's
%-----------------------------------------------------------------------
tol = eps*10;


%-Parse arguments
%-----------------------------------------------------------------------
if nargin<3,     error('insufficient arguments'), end
if nargin<7,	 hReg=[]; else, hReg = varargin{7}; end
if nargin<6,     Title = '';
	else,    Title = varargin{6}; end
if isempty(Title), Title = ['volume summary',...
			' (labels and percentages per cluster)']; end
if nargin<5,     Num = [];
	else,    Num = varargin{5}; end
if isempty(Num), Num = 03; end
if nargin<4,     Dis = [];
	else,    Dis = varargin{4}; end
if isempty(Dis), Dis = 08; end
% VOL is varargin{3} - Use by reference for speed
% SPM is varargin{2} - Don't copy into local variables


%-Get current location (to highlight selected voxel in table)
%-----------------------------------------------------------------------
%xyzmm = spm_results_ui('GetCoords'); % a voir

%-Extract data from structures
%-----------------------------------------------------------------------
S         = varargin{3}.S;
R         = varargin{3}.R;
FWHM      = varargin{3}.FWHM;
v2r       = 1/prod(FWHM);			%-voxels to resels
n         = varargin{2}.n;
STAT      = varargin{2}.STAT;
df        = varargin{2}.df;
u         = varargin{2}.u;
k         = varargin{2}.k*v2r;


%-Setup graphics pane
%-----------------------------------------------------------------------
spm('Pointer','Watch')
Fgraph    = spm_figure('GetWin','Graphics');
spm_results_ui('Clear',Fgraph)
FS        = spm('FontSizes');			%-Scaled font sizes
PF        = spm_platform('fonts');		%-Font names (for this platform)


%-Table header & footer
%=======================================================================

%-Table axes & Title
%-----------------------------------------------------------------------
if isempty(hReg)
hAx   = axes('Position',[0.05 0.1 0.9 0.8],...
	'DefaultTextFontSize',FS(8),...
	'DefaultTextInterpreter','Tex',...
	'DefaultTextVerticalAlignment','Baseline',...
	'Units','points',...
	'Visible','off');
else
hAx   = axes('Position',[0.05 0.1 0.9 0.4],...
	'DefaultTextFontSize',FS(8),...
	'DefaultTextInterpreter','Tex',...
	'DefaultTextVerticalAlignment','Baseline',...
	'Units','points',...
	'Visible','off');
end;
AxPos = get(hAx,'Position'); set(hAx,'YLim',[0,AxPos(4)])
dy    = FS(9);
y     = floor(AxPos(4)) - dy;

text(0,y,['Working Dir : ',varargin{2}.swd],'FontSize',FS(11),'FontWeight','Bold','Editing','on');	
y = y - 1.5*dy;
text(0,y,varargin{2}.title,'FontSize',FS(11),'FontWeight','Bold');	
y = y - 1.5*dy;
text(0,y,['Labels : \it\fontsize{',num2str(FS(9)),'}',Title],...
	'FontSize',FS(11),'FontWeight','Bold');	
y = y - dy/2;
line([0 1],[y y],'LineWidth',3,'Color','r');
y = y - 9*dy/8;


%-Construct table header
%-----------------------------------------------------------------------
set(gca,'DefaultTextFontName',PF.helvetica,'DefaultTextFontSize',FS(8))

Hc = [];

text(0.20,y - dy/2,'x,y,z {mm}','FontSize',FS(9));
text(0.40,y - dy/2,'label','FontSize',FS(9))
text(0.60,y - dy/2,'%','FontSize',FS(9));

label='nom du label';

%-Headers for text table...
TabDat.tit = Title;

TabDat.hdr = {	'',		'x,y,z {mm}';...
				'',		label;...
				'',		'%'}';

TabDat.fmt = { '%3.0f %3.0f %3.0f','%s','%3.2f'};

y     = y - 7*dy/4;
line([0 1],[y y],'LineWidth',1,'Color','r');
y     = y - 5*dy/4;
y0    = y;


%-Table filtering note
%-----------------------------------------------------------------------
if isinf(Num)
	TabDat.str = sprintf('table shows first local maximum per cluster.');
else
	%TabDat.str = sprintf(['table shows at most %s local maxima ',...
	%	'> %.1fmm apart per cluster'],Num,Dis);
	TabDat.str = sprintf('table shows first local maximum per cluster.');
end
text(0.5,4,TabDat.str,'HorizontalAlignment','Center','FontName',PF.helvetica,...
    'FontSize',FS(8),'FontAngle','Italic')


%-Volume, resels and smoothness 
%-----------------------------------------------------------------------
FWHMmm          = FWHM.*varargin{3}.VOX'; 			% FWHM {mm}
Pz              = spm_P(1,0,u,df,STAT,1,n);
Pu              = spm_P(1,0,u,df,STAT,R,n);
[P Pn Em En EN] = spm_P(1,k,u,df,STAT,R,n);


%-Footnote with SPM parameters
%-----------------------------------------------------------------------
line([0 1],[0 0],'LineWidth',1,'Color','r')
set(gca,'DefaultTextFontName',PF.helvetica,...
	'DefaultTextInterpreter','None','DefaultTextFontSize',FS(8))
TabDat.ftr = cell(4,2);
TabDat.ftr{1} = ...
	sprintf('Height threshold: %c = %0.2f, p = %0.3f (%0.3f corrected)',...
		 STAT,u,Pz,Pu);
TabDat.ftr{2} = ...
sprintf('Extent threshold: k = %0.0f voxels, p = %0.3f (%0.3f corrected)',...
	         k/v2r,Pn,P);
TabDat.ftr{3} = ...
	sprintf('Expected voxels per cluster, <k> = %0.3f',En/v2r);
TabDat.ftr{4} = ...
	sprintf('Expected number of clusters, <c> = %0.2f',Em*Pn);
TabDat.ftr{5} = ...
	sprintf('Degrees of freedom = [%0.1f, %0.1f]',df);
TabDat.ftr{6} = ...
	sprintf(['Smoothness FWHM = %0.1f %0.1f %0.1f {mm} ',...
		 ' = %0.1f %0.1f %0.1f {voxels}'],FWHMmm,FWHM);
TabDat.ftr{7} = ...
	sprintf('Search volume: S = %0.0f mm^3 = %0.0f voxels = %0.1f resels',...
		S*prod(varargin{3}.VOX),S,R(end));
TabDat.ftr{8} = ...
	sprintf(['Voxel size: [%0.1f, %0.1f, %0.1f] mm ',...
		' (1 resel = %0.2f voxels)'],...
		varargin{3}.VOX,prod(FWHM));

text(0.0,-1*dy,TabDat.ftr{1},...
	'UserData',[u,Pz,Pu],'ButtonDownFcn','get(gcbo,''UserData'')')
text(0.0,-2*dy,TabDat.ftr{2},...
	'UserData',[k/v2r,Pn,P],'ButtonDownFcn','get(gcbo,''UserData'')')
text(0.0,-3*dy,TabDat.ftr{3},...
	'UserData',En/v2r,'ButtonDownFcn','get(gcbo,''UserData'')')
text(0.0,-4*dy,TabDat.ftr{4},...
	'UserData',Em*Pn,'ButtonDownFcn','get(gcbo,''UserData'')')
text(0.5,-1*dy,TabDat.ftr{5},...
	'UserData',df,'ButtonDownFcn','get(gcbo,''UserData'')')
text(0.5,-2*dy,TabDat.ftr{6},...
	'UserData',FWHMmm,'ButtonDownFcn','get(gcbo,''UserData'')')
text(0.5,-3*dy,TabDat.ftr{7},...
	'UserData',[S*prod(varargin{3}.VOX),S,R(end)],...
	'ButtonDownFcn','get(gcbo,''UserData'')')
text(0.5,-4*dy,TabDat.ftr{8},...
	'UserData',[varargin{3}.VOX',prod(FWHM)],...
	'ButtonDownFcn','get(gcbo,''UserData'')')




%-Characterize excursion set in terms of maxima
% (sorted on Z values and grouped by regions)
%=======================================================================
if ~length(varargin{2}.Z)
	text(0.5,y-6*dy,'no suprathreshold clusters',...
		'HorizontalAlignment','Center',...
		'FontAngle','Italic','FontWeight','Bold',...
		'FontSize',FS(16),'Color',[1,1,1]*.5);
	varargout = {TabDat};
	spm('Pointer','Arrow')
	return
end

[N Z XYZ A] = spm_max(varargin{2}.Z,varargin{2}.XYZ);

%-Convert cluster sizes from voxels to resels
%-----------------------------------------------------------------------
if isfield(varargin{3},'VRVP')
	V2R = spm_sample_vol(varargin{3}.VRVP,XYZ(1,:),XYZ(2,:),XYZ(3,:),0);
else
	V2R = v2r;
end
N         = N.*V2R;

%-Convert maxima locations from voxels to mm
%-----------------------------------------------------------------------
XYZmm     = varargin{3}.M(1:3,:)*[XYZ; ones(1,size(XYZ,2))];

% load MNI ROIs & Labels
%-----------------------------------------------------------------------
	MNI=spm_get(1,'.img','select an labelised atlas');
	MNID=spm_vol(MNI);
	eval(['load ',strrep(MNI,'.img','_List.mat')]);
	%ROI.ID
	%ROI.Nom_C
	%ROI.Nom_L


%-Table proper (& note all data in cell array)
%=======================================================================

%-Pagination variables
%-----------------------------------------------------------------------
hPage = [];
set(gca,'DefaultTextFontName',PF.courier,'DefaultTextFontSize',FS(7))


%-Set-level p values {c} - do not display if reporting a single cluster
%-----------------------------------------------------------------------
c     = max(A);					%-Number of clusters
TabLin     = 1;					%-Table data line


%-Local maxima p-values labels & ances
%-----------------------------------------------------------------------
HlistXYZ = [];
while max(Z)

	% Paginate if necessary
	%---------------------------------------------------------------
	if y < min(Num+1,3)*dy
		h     = text(0.5,-5*dy,...
			sprintf('Page %d',spm_figure('#page')),...
			'FontName',PF.helvetica,'FontAngle','Italic',...
			'FontSize',FS(8));
		spm_figure('NewPage',[hPage,h])
		hPage = [];
		y     = y0;
	end

    	%-Find largest remaining local maximum
    	%---------------------------------------------------------------
	[U,i]   = max(Z);			% largest maxima
	j       = find(A == A(i));		% maxima in cluster

	%-correction le 25/04/01
	%-Jump to voxel nearest current location
	[xyzmm,ii] = spm_XYZreg('NearestXYZ', XYZmm(:,i), varargin{2}.XYZmm);
	%-Find selected cluster
	B         = spm_clusters(varargin{2}.XYZ);
	jj        = find(B == B(ii));
	%-fin correction le 25/04/01

	loc	    = varargin{2}.XYZmm(:,jj); 
	
	%-Compute Labels & Percent
    %---------------------------------------------------------------
		[Label,Perc] = gin_det_plabels(loc, MNID, ROI);

		h     = text(0.18,y,sprintf(TabDat.fmt{1},XYZmm(:,i)),...
			'FontWeight','Bold',...
			'Tag','ListXYZ',...
			'ButtonDownFcn',...
			'spm_mip_ui(''SetCoords'',get(gcbo,''UserData''));',...
			'Interruptible','off','BusyAction','Cancel',...
			'UserData',XYZmm(:,i));
		HlistXYZ = [HlistXYZ, h];
		hPage = [hPage, h];
 
 		for kkk=size(Perc,2):-1:1,
			if Perc(kkk) >= 1.0
			if y < 2*dy
				h = text(0.5,-5*dy,sprintf('Page %d',...
					spm_figure('#page')),...
					'FontName',PF.helvetica,...
					'FontAngle','Italic',...
					'FontSize',FS(8));
				spm_figure('NewPage',[hPage,h])
				hPage = [];
				y     = y0;
			end

			h     = text(0.37,y,sprintf(TabDat.fmt{2},Label(kkk).Nom),	'FontWeight','Bold',...
				'UserData',label,'ButtonDownFcn','get(gcbo,''UserData'')');
			hPage = [hPage, h];

			h     = text(0.60,y,sprintf(TabDat.fmt{3},Perc(kkk)),	'FontWeight','Bold',...
				'UserData',Perc(kkk),'ButtonDownFcn','get(gcbo,''UserData'')');
			hPage = [hPage, h];

			y = y - dy;
			end
		end
    %---------------------------------------------------------------

	TabLin = TabLin + 1;
	
	Z(j) = Z(j)*0;		% Zero local maxima for this cluster
end						% end region


%-Number and register last page (if paginated)
%-----------------------------------------------------------------------
if spm_figure('#page')>1
	h = text(0.5,-5*dy,sprintf('Page %d/%d',spm_figure('#page')*[1,1]),...
		'FontName',PF.helvetica,'FontSize',FS(8),'FontAngle','Italic');
	spm_figure('NewPage',[hPage,h])
end



%-End: Store TabDat in UserData of axes & reset pointer
%=======================================================================
h      = uicontextmenu('Tag','TabDat',...
		'UserData',TabDat);
set(gca,'UIContextMenu',h,...
	'Visible','on',...
	'XColor','w','YColor','w')
uimenu(h,'Label','Table')
uimenu(h,'Separator','on','Label','Print text table',...
	'Tag','TD_TxtTab',...
	'CallBack',...
	'gin_clusters_plabels(''txtlist'',get(get(gcbo,''Parent''),''UserData''),3)',...
	'Interruptible','off','BusyAction','Cancel');
uimenu(h,'Separator','off','Label','Extract table data structure',...
	'Tag','TD_Xdat',...
	'CallBack','get(get(gcbo,''Parent''),''UserData'')',...
	'Interruptible','off','BusyAction','Cancel');
uimenu(h,'Separator','on','Label','help',...
	'Tag','TD_Xdat',...
	'CallBack','spm_help(''gin_clusters_plabels'')',...
	'Interruptible','off','BusyAction','Cancel');

%-Setup registry
%-----------------------------------------------------------------------
set(hAx,'UserData',struct('hReg',hReg,'HlistXYZ',HlistXYZ))
%spm_XYZreg('Add2Reg',hReg,hAx,'spm_list');

%-Return TabDat structure & reset pointer
%-----------------------------------------------------------------------
varargout = {TabDat};
spm('Pointer','Arrow')


